@@ -53,14 +53,14 @@ INSTALL_SCHEMES = {
     'unix_local': {
         'purelib': '$base/local/lib/python$py_version_short/dist-packages',
         'platlib': '$platbase/local/lib/python$py_version_short/dist-packages',
-        'headers': '$base/local/include/python$py_version_short$abiflags/$dist_name',
+        'headers': '$base/local/include/python$py_version_short/$dist_name',
         'scripts': '$base/local/bin',
         'data'   : '$base/local',
         },
     'deb_system': {
         'purelib': '$base/lib/python3/dist-packages',
         'platlib': '$platbase/lib/python3/dist-packages',
-        'headers': '$base/include/python$py_version_short$abiflags/$dist_name',
+        'headers': '$base/include/python$py_version_short/$dist_name',
         'scripts': '$base/bin',
         'data'   : '$base',
         },
@@ -299,8 +299,8 @@ class install(Command):
 
         if self.user and (self.prefix or self.exec_prefix or self.home or
                 self.install_base or self.install_platbase):
-            raise DistutilsOptionError("can't combine user with with prefix/"
-                                       "exec_prefix/home or install_(plat)base")
+            raise DistutilsOptionError("can't combine user with prefix, "
+                                       "exec_prefix/home, or install_(plat)base")
 
         # Next, stuff that's wrong (or dubious) only on certain platforms.
         if os.name != "posix":
@@ -487,6 +487,7 @@ class install(Command):
                         "unknown value for --install-layout")
             elif (self.prefix_option and os.path.normpath(self.prefix) != '/usr/local') \
                     or 'PYTHONUSERBASE' in os.environ \
+                    or 'VIRTUAL_ENV' in os.environ \
                     or 'real_prefix' in sys.__dict__:
                 self.select_scheme("unix_prefix")
             else:
